(define-module (ordo action remote-cmd)
  #:use-module (ice-9 exceptions)
  #:use-module (srfi srfi-1)
  #:use-module (srfi srfi-71)
  #:use-module (ordo connection)
  #:use-module (ordo connection base)
  #:use-module (ordo logger)
  #:use-module (ordo util flatten)
  #:use-module (ordo util keyword-args)
  #:export (remote-cmd))

(define (remote-cmd conn prog . args)
  (let* ((args options (break keyword? args))
         (args (remove unspecified? (flatten args)))
         (return (keyword-arg options #:return identity))
         (check? (keyword-arg options #:check?))
         (command (build-command conn prog args options)))
    (log-msg 'DEBUG "Running command: " command " on connection " (describe conn))
    (let ((out rc (remote-exec conn command)))
      (log-msg 'DEBUG "Command exit code: " rc)
      (if check?
          (if (zero? rc)
              (return out)
              (raise-exception (make-exception
                                (make-external-error)
                                (make-exception-with-message (format #f "Non-zero exit (~a) from ~a" rc prog)))))
          (values (return out) rc)))))
